﻿using Hotline.FlowEngine.Definitions;
using Hotline.Share.Enums.FlowEngine;
using SqlSugar;
using XF.Domain.Repository;

namespace Hotline.FlowEngine.Workflows;

public class Workflow : CreationEntity
{
    public string DefinitionId { get; set; }

    /// <summary>
    /// 到期时间
    /// </summary>
    public DateTime ExpiredTime { get; set; }

    public DateTime? CompleteTime { get; set; }

    public EWorkflowStatus Status { get; set; }

    /// <summary>
    /// 当前节点名称（会签状态此字段保存开启会签时办理节点名称）
    /// </summary>
    public string CurrentStepName { get; set; }

    /// <summary>
    /// 到达当前节点时间（stepBox创建时间）
    /// </summary>
    public DateTime CurrentStepTime { get; set; }

    /// <summary>
    /// 当前发起会签节点code（不处于会签状态则无值）
    /// </summary>
    public string? CurrentCountersignCode { get; set; }


    [Navigate(NavigateType.OneToOne, nameof(DefinitionId))]
    public Definition Definition { get; set; }

    /// <summary>
    /// 补充信息
    /// </summary>
    [Navigate(NavigateType.OneToMany, nameof(WorkflowSupplement.WorkflowId))]
    public List<WorkflowSupplement> Supplements { get; set; }

    /// <summary>
    /// 主节点，依据流转进度动态生成
    /// </summary>
    [SugarColumn(IsIgnore = true)]
    public List<WorkflowStep> StepBoxes { get; set; }

    [SugarColumn(IsIgnore = true)]
    public List<WorkflowTrace> Traces { get; set; } = new();

    public void Complete()
    {
        Status = EWorkflowStatus.Completed;
        CompleteTime = DateTime.Now;
    }

    /// <summary>
    /// 流程进行流转
    /// </summary>
    public void FLow(string currentStepName, DateTime currentStepTime, string? currentCountersignCode = null)
    {
        CurrentStepName = currentStepName;
        CurrentStepTime = currentStepTime;
        if (!string.IsNullOrEmpty(currentCountersignCode))
            CurrentCountersignCode = currentCountersignCode;
    }

    /// <summary>
    /// 检查会签是否结束，并更新当前会签节点
    /// </summary>
    public void CheckCountersignAndUpdate()
    {
        if (IsInCountersign())
        {
            var countersignStepBox = StepBoxes.First(d => d.Code == CurrentCountersignCode);
            var countersignOver = countersignStepBox.Steps.All(d => d.Status is EWorkflowStepStatus.Completed);//todo 有bug，考虑会签嵌套的场景
            if (countersignOver) //会签结束
                CurrentCountersignCode = null;
        }
    }

    public void CheckCurrentStepBoxCompleted()
    {

    }

    /// <summary>
    /// 流程是否处于会签中
    /// </summary>
    /// <returns></returns>
    public bool IsInCountersign() => !string.IsNullOrEmpty(CurrentCountersignCode);

    /// <summary>
    /// 结束流程会签状态
    /// </summary>
    public void CloseCountersignStatus() => CurrentCountersignCode = null;
}